@@ -18,6 +18,7 @@
  */
 
 #include <stdlib.h>
+#include <locale.h>
 #include "config.h"
 #ifdef HAVE_GETOPT_H
 #  include <getopt.h>
@@ -39,6 +40,12 @@ static void build_list1 (gpointer data, GList ** list)
   *list = g_list_prepend (*list, data);
 }
 
+static void build_list2 (GtsVertex * v, GList ** list)
+{
+  if (gts_vertex_is_boundary (v, NULL))
+    *list = g_list_prepend (*list, v);
+}
+
 static void vertex_cleanup (GtsVertex * v)
 {
   gts_vertex_is_contact (v, TRUE);
@@ -110,6 +117,12 @@ static void triangle_cleanup (GtsSurface * s)
   g_slist_free (triangles);
 }
 
+static gboolean check_boundaries (GtsVertex * v1, GtsVertex * v2)
+{
+  return (g_slist_length (v1->segments) < 4 && 
+	  g_slist_length (v2->segments) < 4);
+}
+
 /* cleanup - using a given threshold merge vertices which are too close.
    Eliminate degenerate and duplicate edges.
    Eliminate duplicate triangles . */
@@ -117,10 +130,14 @@ int main (int argc, char * argv[])
 {
   GtsSurface * s, * m;
   GList * vertices = NULL;
-  gboolean verbose = FALSE, sever = FALSE;
+  gboolean verbose = FALSE, sever = FALSE, boundary = FALSE;
   gdouble threshold;
   int c = 0;
   GtsFile * fp;
+  gboolean (* check) (GtsVertex *, GtsVertex *) = NULL;
+
+  if (!setlocale (LC_ALL, "POSIX"))
+    g_warning ("cannot set locale to POSIX");
 
   s = gts_surface_new (gts_surface_class (),
 		       gts_face_class (),
@@ -131,17 +148,25 @@ int main (int argc, char * argv[])
   while (c != EOF) {
 #ifdef HAVE_GETOPT_LONG
     static struct option long_options[] = {
+      {"2D", no_argument, NULL, 'c'},
+      {"boundary", no_argument, NULL, 'b'},
       {"merge", required_argument, NULL, 'm'},
       {"sever", no_argument, NULL, 's'},
       {"help", no_argument, NULL, 'h'},
       {"verbose", no_argument, NULL, 'v'}
     };
     int option_index = 0;
-    switch ((c = getopt_long (argc, argv, "hvsm:", 
+    switch ((c = getopt_long (argc, argv, "hvsm:bc", 
 			      long_options, &option_index))) {
 #else /* not HAVE_GETOPT_LONG */
-    switch ((c = getopt (argc, argv, "hvsm:"))) {
+    switch ((c = getopt (argc, argv, "hvsm:bc"))) {
 #endif /* not HAVE_GETOPT_LONG */
+    case 'c': /* 2D */
+      check = check_boundaries;
+      break;
+    case 'b': /* boundary */
+      boundary = TRUE;
+      break;
     case 's': /* sever */
       sever = TRUE;
       break;
@@ -157,7 +182,7 @@ int main (int argc, char * argv[])
       m = gts_surface_new (gts_surface_class (),
 			   gts_face_class (),
 			   gts_edge_class (),
-			   gts_vertex_class ());
+			   s->vertex_class);
       fp = gts_file_new (fptr);
       if (gts_surface_read (m, fp)) {
 	fprintf (stderr, "cleanup: file `%s' is not a valid GTS file\n", 
@@ -177,17 +202,19 @@ int main (int argc, char * argv[])
       break;
     case 'h': /* help */
       fprintf (stderr,
-             "Usage: cleanup [OPTION] THRESHOLD < FILE\n"
-	     "Merge vertices of the GTS surface FILE if they are closer than THRESHOLD,\n"
-	     "eliminate degenerate, duplicate edges and duplicate triangles.\n"
-	     "\n"
-	     "  -s      --sever    sever \"contact\" vertices\n"
-	     "  -m FILE --merge    merge surface FILE\n"
-	     "  -v      --verbose  print statistics about the surface\n"
-	     "  -h      --help     display this help and exit\n"
-	     "\n"
-	     "Report bugs to %s\n",
-	     GTS_MAINTAINER);
+	       "Usage: cleanup [OPTION] THRESHOLD < FILE\n"
+	       "Merge vertices of the GTS surface FILE if they are closer than THRESHOLD,\n"
+	       "eliminate degenerate, duplicate edges and duplicate triangles.\n"
+	       "\n"
+	       "  -c      --2D        2D boundary merging\n"
+	       "  -b      --boundary  only consider boundary vertices for merging\n"
+	       "  -s      --sever     sever \"contact\" vertices\n"
+	       "  -m FILE --merge     merge surface FILE\n"
+	       "  -v      --verbose   print statistics about the surface\n"
+	       "  -h      --help      display this help and exit\n"
+	       "\n"
+	       "Report bugs to %s\n",
+	       GTS_MAINTAINER);
       return 0; /* success */
       break;
     case '?': /* wrong options */
@@ -197,7 +224,7 @@ int main (int argc, char * argv[])
   }
 
   if (optind >= argc) { /* missing threshold */
-    fprintf (stderr, 
+    fprintf (stderr,
 	     "cleanup: missing THRESHOLD\n"
 	     "Try `cleanup --help' for more information.\n");
     return 1; /* failure */
@@ -216,7 +243,7 @@ int main (int argc, char * argv[])
   m = gts_surface_new (gts_surface_class (),
 		       gts_face_class (),
 		       gts_edge_class (),
-		       gts_vertex_class ());
+		       s->vertex_class);
   fp = gts_file_new (stdin);
   if (gts_surface_read (m, fp)) {
     fputs ("cleanup: file on standard input is not a valid GTS file\n", 
@@ -228,15 +255,16 @@ int main (int argc, char * argv[])
   gts_object_destroy (GTS_OBJECT (m));
 
   /* if verbose on print stats */
-  if (verbose) 
+  if (verbose)
     gts_surface_print_stats (s, stderr);
  
   /* merge vertices which are close enough */
   /* build list of vertices */
-  gts_surface_foreach_vertex (s, (GtsFunc) build_list1, &vertices);
+  gts_surface_foreach_vertex (s, boundary ? (GtsFunc) build_list2 : (GtsFunc) build_list1,
+			      &vertices);
   /* merge vertices: we MUST update the variable vertices because this function
      modifies the list (i.e. removes the merged vertices). */
-  vertices = gts_vertices_merge (vertices, threshold);
+  vertices = gts_vertices_merge (vertices, threshold, check);
 
   /* free the list */
   g_list_free (vertices);
