@@ -259,8 +259,8 @@ GtsVertex * gts_volume_optimized_vertex (GtsEdge * edge,
   g_return_val_if_fail (klass != NULL, NULL);
   g_return_val_if_fail (params != NULL, NULL);
 
-  A = gts_matrix_new (0., 0., 0., 0., 0., 0., 0., 0., 0.);
-  Hb = gts_matrix_new (0., 0., 0., 0., 0., 0., 0., 0., 0.);
+  A = gts_matrix_zero (NULL);
+  Hb = gts_matrix_zero (NULL);
   v1 = GTS_SEGMENT (edge)->v1;
   v2 = GTS_SEGMENT (edge)->v2;
 
@@ -290,9 +290,10 @@ GtsVertex * gts_volume_optimized_vertex (GtsEdge * edge,
   }
   if (nb > 0) {
     GtsMatrix * H = gts_matrix_new (
-	       e1[2]*e1[2] + e1[1]*e1[1], - e1[0]*e1[1], - e1[0]*e1[2],
-	     - e1[0]*e1[1], e1[2]*e1[2] + e1[0]*e1[0], - e1[1]*e1[2],
-	     - e1[0]*e1[2], - e1[1]*e1[2], e1[1]*e1[1] + e1[0]*e1[0]);
+	       e1[2]*e1[2] + e1[1]*e1[1], - e1[0]*e1[1], - e1[0]*e1[2], 0.,
+	       - e1[0]*e1[1], e1[2]*e1[2] + e1[0]*e1[0], - e1[1]*e1[2], 0.,
+	       - e1[0]*e1[2], - e1[1]*e1[2], e1[1]*e1[1] + e1[0]*e1[0], 0.,
+	       0., 0., 0., 0.);
     GtsVector c;
 
     c[0] = e1[1]*e2[2] - e1[2]*e2[1];
@@ -345,9 +346,10 @@ GtsVertex * gts_volume_optimized_vertex (GtsEdge * edge,
 #if 1 /* Weighted average of volume and boundary optimization */
   if (n < 3) {
     /* volume optimization and boundary optimization */
-    GtsMatrix * H = gts_matrix_new (sn11, sn12, sn13,
-				    sn12, sn22, sn23,
-				    sn13, sn23, sn33);
+    GtsMatrix * H = gts_matrix_new (sn11, sn12, sn13, 0.,
+				    sn12, sn22, sn23, 0.,
+				    sn13, sn23, sn33, 0.,
+				    0., 0., 0., 0.);
     GtsVector c;
     gdouble le = 9.*params->boundary_weight*
       gts_point_distance2 (GTS_POINT (v1), 
@@ -397,9 +399,10 @@ GtsVertex * gts_volume_optimized_vertex (GtsEdge * edge,
     }
     g_slist_free (list);
     
-    H = gts_matrix_new (nv, 0., 0.,
-			0., nv, 0.,
-			0., 0., nv);
+    H = gts_matrix_new (nv, 0., 0., 0.,
+			0., nv, 0., 0.,
+			0., 0., nv, 0.,
+			0., 0., 0., 0.);
     n = gts_matrix_quadratic_optimization (A, b, n, H, c);
     gts_matrix_destroy (H);
   }
@@ -472,7 +475,7 @@ GtsVertex * gts_volume_optimized_vertex (GtsEdge * edge,
 #endif /* Weighted average of volume, boundary and shape optimization */
 
   g_assert (n == 3);
-  g_assert ((Ai = gts_matrix_inverse (A)));
+  g_assert ((Ai = gts_matrix3_inverse (A)));
 
   v = gts_vertex_new (klass,
 		      Ai[0][0]*b[0] + Ai[0][1]*b[1] + Ai[0][2]*b[2],
