@@ -19,6 +19,7 @@
 
 #include <math.h>
 #include <stdlib.h>
+#include <locale.h>
 #include <string.h>
 #include "config.h"
 #ifdef HAVE_GETOPT_H
@@ -29,6 +30,11 @@
 #endif /* HAVE_UNISTD_H */
 #include "gts.h"
 
+#ifdef NATIVE_WIN32
+#  include <fcntl.h>
+#  include <io.h>
+#endif 
+
 static GPtrArray * stl_read (FILE * fp)
 {
   GPtrArray * a = g_ptr_array_new ();
@@ -86,6 +92,10 @@ static GPtrArray * stl_read (FILE * fp)
     gchar header[80];
     guint i;
 
+#ifdef NATIVE_WIN32
+    _setmode ( _fileno (fp), _O_BINARY);
+#endif 
+
     if (fread (header, sizeof (gchar), 80, fp) != 80) {
       fprintf (stderr, "Input file is not a valid STL file\n"
 	       "stdin: incomplete header\n");
@@ -142,12 +152,6 @@ static GPtrArray * stl_read (FILE * fp)
 	       "stdin: missing attribute byte count\n");
 	exit (1);
       }
-      if (attbytecount != 0) {
-	fprintf (stderr, "Input file is not a valid STL file\n"
-	       "stdin: attribute byte count is not zero\n");
-	exit (1);
-      }
-      
       i--;
     }
   }
@@ -251,6 +255,9 @@ int main (int argc, char * argv[])
   GPtrArray * stl;
   GtsSurface * s;
 
+  if (!setlocale (LC_ALL, "POSIX"))
+    g_warning ("cannot set locale to POSIX");
+
   while (c != EOF) {
 #ifdef HAVE_GETOPT_LONG
     static struct option long_options[] = {
